/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef D22_AUTOCONFIG_H
#define D22_AUTOCONFIG_H

#include <Controller.h>
#include <cameo/cameo.h>
#include <controllers/lss/d22/D22Settings.h>
#include <controllers/common/utilities/CountSpy.h>
#include <controllers/lss/remotelsssimulation/RemoteLSSSimulation.h>
#include <drivers/utilities/null/NullDriver.h>

namespace d22 {

class AutoConfig : public ExperimentController,
						public controller::Stoppable {

public:
	//! Type of controller
	static const std::string TYPE;

	AutoConfig(const std::string& name);
	AutoConfig(const AutoConfig& controller);
	virtual ~AutoConfig();

	virtual void postConfiguration();

	void apply();
	virtual void start();
	virtual void stop();

	void updateProgression();

	Property<std::string> serverEndpoint;
	Property<bool> initialized;

	Property<std::string> scatterModelType;
	Property<std::string> scatterModel;
	Property<std::string> scatterModelAccuracy;
	Property<std::string> sampleParameters;

	Property<int32> calcTypeSize;
	DynamicProperty<std::string> calcType;

	Property<int32> resultSize;
	DynamicProperty<std::string> type;
	DynamicProperty<float64> distance;
	DynamicProperty<float64> wavelength;
	DynamicProperty<float64> collimation;

	Property<std::string> wantedType;
	Property<float64> wantedDistance;
	Property<float64> wantedWavelength;
	Property<float64> wantedCollimation;

private:
	bool initApplication();
	float64 getSampleBackground();

	static const std::string REMOTE_APPLICATION;
	static const std::string RESPONDER;

	std::unique_ptr<cameo::Server> m_server;
	std::unique_ptr<cameo::application::Instance> m_remoteApplication;
	std::unique_ptr<cameo::application::Requester> m_requester;

	ControllerPtr<D22Settings> m_d22settings;
	ControllerPtr<remotelsssimulation::RemoteLSSSimulation> m_simulation;
	DriverPtr<driver::NullDriver> m_driver;
	utilities::CountSpy* m_countSpy;
	std::vector<float64> m_collimationValues = { 2.0, 2.8, 4.0, 5.6, 8.0, 11.2, 14.4, 17.6 };
};

}

#endif
