/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include <iostream>
#include <common/base/ServerProperties.h>
#include "controllers/common/family/Families.h"
#include "RemoteNanokin.h"

namespace nanokin {

using namespace std;
using namespace common;
using namespace boost;
using namespace cameo;

const std::string RemoteNanokin::TYPE = "remote_nanokin";

const std::string RemoteNanokin::NANOKIN_CMD = "vascokin";

/*
 * Constructor
 */
RemoteNanokin::RemoteNanokin(const std::string& name) :
		ExperimentController(name), controller::Stoppable(this) {

	setFamily(family::SAMPLE_ENVIRONMENT);

	serverEndpoint.init(this, SAVE, "cameo_server");

	m_driver.init(this, "driver");

	m_initialized = false;
}

RemoteNanokin::RemoteNanokin(const RemoteNanokin& controller) : ExperimentController(controller), controller::Stoppable(this) {
	m_initialized = false;
}

/*
 * Destructor
 */
RemoteNanokin::~RemoteNanokin() {
}

void RemoteNanokin::postConfiguration() {

	if (m_server.get() == 0) {
		// Create a new server if it is not already created.
		// In case of a simulated server, avoid the real remote endpoint.
		if (serverEndpoint() == "" || m_driver->state() != REAL_DEVICE_CONTAINER) {
			m_server.reset(new Server(application::This::getServer().getEndpoint()));
		} else {
			m_server.reset(new Server(serverEndpoint()));
		}
	}
}

/*
 * start
 */
void RemoteNanokin::start() {

	commandProgression = 0;
	commandStatus.setRunning();
	try {
		if (!m_server->isAvailable(10000)) {
			cout << "Nanokin cameo server is not available" << endl;
			commandStatus.setError();
			return;
		}

		cout << "Nanokin cameo server is connected to " << getName() << endl;

		m_StartApplication = m_server->connect(NANOKIN_CMD);

		cout << "connected application" << endl;

		if (m_StartApplication->exists()) {
			// The application exists from a previous server session
			m_StartApplication->kill();
			application::State state = m_StartApplication->waitFor();
			cout << "Terminated start application " << state << endl;
		}

		//start args="START DefaultSOP.sopx  c:\\temp save.dat"
		vector<string> args;
		args.push_back("START");
		args.push_back("DefaultSOP.sopx");
		args.push_back("c:\\temp");
		args.push_back("save.dat");
		m_StartApplication = m_server->start(NANOKIN_CMD, args);

//		if (m_StartApplication->exists()) {
//			cout << "No matlab application" << endl;
//			return false;
//		}

		m_StartApplication->waitFor(application::STOPPED);

	} catch (const std::exception & e) {
		// Currently an exception can occur during isAvailable.
		cout << "Unexpected exception during nanokin cameo connection: " << e.what() << endl;
	}
}

/*
 * stop
 */
void RemoteNanokin::stop() {

	try {
		if (!m_server->isAvailable(10000)) {
			cout << "Nanokin cameo server is not available" << endl;
			return;
		}

		cout << "Nanokin cameo server is connected to " << getName() << endl;

		m_StopApplication = m_server->connect(NANOKIN_CMD);

//		cout << "connected application" << endl;
//
//		if (m_StopApplication->exists()) {
//			// The application exists from a previous server session
//			m_StopApplication->kill();
//			application::State state = m_StopApplication->waitFor();
//			cout << "Terminated stop application " << state << endl;
//		}

		vector<string> args;
		args.push_back("STOP");
		m_StopApplication = m_server->start(NANOKIN_CMD, args);

//		if (m_StartApplication->exists()) {
//			cout << "No matlab application" << endl;
//			return false;
//		}

		m_StartApplication->waitFor(application::STOPPED);

	} catch (const std::exception & e) {
		// Currently an exception can occur during isAvailable.
		cout << "Unexpected exception during nanokin cameo connection: " << e.what() << endl;
	}
	commandProgression = 100;
	commandStatus.setIdle();
}

}
