/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include <cmath>
#include <bitset>

#include "drivers/stereo_elec/StereoElecDef.h"

#include "CalibParamController.h"
#include "controllers/common/family/Families.h"

using namespace std;

namespace stereo_daq {

const string CalibParamController::TYPE = "calibparam_controller";

/*
 * Constructor
 */
CalibParamController::CalibParamController(const string& name) : ExperimentController(name),controller::Stoppable(this),Select(this),Set(this) {

	setFamily(family::ACQUISITION,family::SETTING);


	sourcetype.init(this, NOSAVE, "sourcetype");
	x.init(this, NOSAVE, "x");
	y.init(this, NOSAVE, "y");
	z.init(this, NOSAVE, "z");

	m_ELECDriver.init(this, "driver");

	registerFunction(TYPE);




}

CalibParamController::CalibParamController(const CalibParamController& controller) : ExperimentController(controller),controller::Stoppable(this), Select(this),Set(this) {

	m_ELECDriver.copy(controller.m_ELECDriver);


	sourcetype.copy(this, controller.sourcetype);
	x.copy(this, controller.x);
	y.copy(this, controller.y);
	z.copy(this, controller.z);


}

/*
 * Destructor
 */
CalibParamController::~CalibParamController() {
	// Empty
}

void CalibParamController::postConfiguration() {


	registerRefresher(sourcetype, &CalibParamController::refreshSourceTypeProperty, this);
	registerRefresher(x, &CalibParamController::refreshXProperty, this);
	registerRefresher(y, &CalibParamController::refreshYProperty, this);
	registerRefresher(z, &CalibParamController::refreshZProperty, this);

	sourcetype="none";
	x=0;
	y=0;
	z=0;

}


void CalibParamController::refreshSourceTypeProperty(const std::string& value) throw (CannotSetValue) {
	m_ELECDriver->DAQParams.sourceType = (string) value;
}
void CalibParamController::refreshXProperty(float64 value) throw (CannotSetValue) {
	m_ELECDriver->DAQParams.sourcePosX = (float) value;
}
void CalibParamController::refreshYProperty(float64 value) throw (CannotSetValue) {
	m_ELECDriver->DAQParams.sourcePosY = (float) value;
}
void CalibParamController::refreshZProperty(float64 value) throw (CannotSetValue) {
	m_ELECDriver->DAQParams.sourcePosZ = (float) value;
}



void CalibParamController::select() {


}

void CalibParamController::set() {

}

void CalibParamController::start() {


}

void CalibParamController::stop() {

}


}

