/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#ifndef CAENCFDSTATE_H
#define CAENCFDSTATE_H

#include <Driver.h>
#include "CAENCfdDriver.h"

namespace caen_cfd {

/*!
 * \class CAENCfdState
 * \brief Implements State pattern
 */
class CAENCfdState: public DriverState<CAENCfdDriver> {

public:

	/*!
	 * \brief Constructor.
	 */
	CAENCfdState(CAENCfdDriver* owner) : DriverState<CAENCfdDriver> (owner) {}

	/*!
	 * \brief Destructor.
	 */
	virtual ~CAENCfdState() {}

	/*!
	 * \brief Reset command implementation
	 */
	virtual void reset() = 0;

	/*!
	 * \brief Clear command implementation
	 */
	virtual void clear() = 0;

	/*!
	 * \brief Write Param command implementation
	 */
	virtual void writeParam() = 0;

	/*!
	 * \brief Read Infos command implementation
	 */
	virtual void readInfos() = 0;

	/*!
	 * \brief Read Infos command implementation
	 */
	virtual void readStatus() = 0;

	/*!
	 * \brief Set acquisition mode command implementation
	 */
	virtual void setAcqMode() = 0;

	/*!
	 * \brief Set CFD mode parameters
	 */
	virtual void setCfdParam() = 0;

	/*!
	 * \brief Set bit in register
	 */
	//virtual void RegisterSetBits(uint16_t addr, int32 startBit, int32 endBit, int32 val) = 0;
};

}

#endif //CAENCFDSTATE_H
