/*
 * Nomad Instrument Control Software
 *
 * Copyright 2011 Institut Laue-Langevin
 *
 * Licensed under the EUPL, Version 1.1 only (the "License");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 */

#include "RealCAENSy2527BoardDriver.h"

#include "drivers/global/DriversCommands.h"
#include "CAENSy2527BoardDriver.h"
#include "CAENSy2527Def.h"
#include "CAENSy2527Driver.h"

using namespace std;

namespace caen_sy2527 {

/*
 * Constructor
 */
RealCAENSy2527BoardDriver::RealCAENSy2527BoardDriver(CAENSy2527BoardDriver* owner) :
	CAENSy2527BoardState(owner) {

	// Empty
}

/*
 * Destructor
 */
RealCAENSy2527BoardDriver::~RealCAENSy2527BoardDriver() {

	// Empty
}

/*
 * init
 */
void RealCAENSy2527BoardDriver::init() {

	CAENSy2527Driver* driver = dynamic_cast<CAENSy2527Driver *> (owner()->getParent());

	owner()->boardType = driver->boardModel(owner()->leafChannel()-1);
	owner()->nbChannels = driver->channelsPerBoard(owner()->leafChannel()-1);

	int32 channels = owner()->nbChannels();
	int32 board = owner()->leafChannel()-1;

	owner()->i0Set.resize(channels);
	owner()->v0Set.resize(channels);
	owner()->vMin.resize(channels);
	owner()->vMax.resize(channels);
	owner()->iMon.resize(channels);
	owner()->vMon.resize(channels);
	owner()->rampUp.resize(channels);
	owner()->rampDown.resize(channels);
	owner()->channelName.resize(channels);
	owner()->power.resize(channels);
	owner()->channelStatus.resize(channels);

	CAENHVRESULT ret;

	unsigned short chList[channels];
	char  		   listNameCh[channels][MAX_CH_NAME];
	float		   fParValList[channels];
	int32   	   iParValList[channels];

	if(channels > 0){

		for (int32 i=0; i<channels; ++i) {
			chList[i] = (unsigned short) i;
		}

		ret = CAENHV_GetChName(driver->m_crateHandle, board, channels, chList, listNameCh);

		if( ret == CAENHV_OK) {
			for (int32 i= 0; i<channels; ++i  ){
				owner()->channelName.set(i, listNameCh[i]);
			}
		} else {
			cout << "Error reading board info" << endl;
		}

		ret = CAENHV_GetChParam(driver->m_crateHandle, board, "V0Set", channels, chList, fParValList);

		if( ret == CAENHV_OK) {
			for (int32 i= 0; i<channels; ++i ) {
				owner()->v0Set.set(i, fParValList[i]);
			}
		} else {
			cout << "Error reading board info" << endl;
		}

		ret = CAENHV_GetChParam(driver->m_crateHandle, board, "I0Set", channels, chList, fParValList);

		if( ret == CAENHV_OK) {
			for (int32 i= 0; i<channels; ++i ) {
				owner()->i0Set.set(i, fParValList[i]);
			}
		} else {
			cout << "Error reading board info" << endl;
		}

		ret = CAENHV_GetChParam(driver->m_crateHandle, board, "RUp", channels, chList, fParValList);

		if( ret == CAENHV_OK) {
			for (int32 i= 0; i<channels; ++i ) {
				owner()->rampUp.set(i, fParValList[i]);
			}
		} else {
			cout << "Error reading board info" << endl;
		}

		ret = CAENHV_GetChParam(driver->m_crateHandle, board, "RDWn", channels, chList, fParValList);

		if( ret == CAENHV_OK) {
			for (int32 i= 0; i<channels; ++i ) {
				owner()->rampDown.set(i, fParValList[i]);
			}
		} else {
			cout << "Error reading board info" << endl;
		}

		ret = CAENHV_GetChParam(driver->m_crateHandle, board, "Pw", channels, chList, iParValList);

		if( ret == CAENHV_OK) {
			for (int32 i= 0; i<channels; ++i ) {
				if(iParValList[i] != 0) owner()->power.set(i, "On");
				else  owner()->power.set(i, "Off");
			}
		} else {
			cout << "Error reading board info" << endl;
		}
	}

	readParam();
	readStatus();
}

/*
 * readStatus
 */
void RealCAENSy2527BoardDriver::readStatus() {

	CAENSy2527Driver* driver = dynamic_cast<CAENSy2527Driver *> (owner()->getParent());
	driver->readStatus(owner());
}

/*
 * read
 */
void RealCAENSy2527BoardDriver::read() {

	CAENSy2527Driver* driver = dynamic_cast<CAENSy2527Driver *> (owner()->getParent());
	driver->read(owner());
}

/*
 * readParam
 */
void RealCAENSy2527BoardDriver::readParam() {

	CAENSy2527Driver* driver = dynamic_cast<CAENSy2527Driver *> (owner()->getParent());
	driver->readParam(owner());
}

/*
 * writeParam
 */
void RealCAENSy2527BoardDriver::writeParam() {

	CAENSy2527Driver* driver = dynamic_cast<CAENSy2527Driver *> (owner()->getParent());
	driver->writeParam(owner());
}

/*
 * start
 */
void RealCAENSy2527BoardDriver::start() {

	CAENSy2527Driver* driver = dynamic_cast<CAENSy2527Driver *> (owner()->getParent());
	driver->writeParam(owner());
	driver->start(owner());
}

/*
 * stop
 */
void RealCAENSy2527BoardDriver::stop() {

	CAENSy2527Driver* driver = dynamic_cast<CAENSy2527Driver *> (owner()->getParent());
	driver->stop(owner());
}

/*
 * powerOn
 */
void RealCAENSy2527BoardDriver::powerOn() {

	CAENSy2527Driver* driver = dynamic_cast<CAENSy2527Driver *> (owner()->getParent());
	driver->powerOn(owner());
}

/*
 * powerOff
 */
void RealCAENSy2527BoardDriver::powerOff() {

	CAENSy2527Driver* driver = dynamic_cast<CAENSy2527Driver *> (owner()->getParent());
	driver->powerOff(owner());
}

}
